#!/bin/bash
set -e
set -x

if [ $# -eq 0 ]; then
    echo "No arguments provided"
    exit 1
fi

env_file=$1
model_name=$2

source ${env_file}

curr_timestamp=$(date +"%Y%m%d%H%M%S")
results_dir=${WS_PATH}/exp_results/$model_name
log_file=${SS_LOGS_PATH}/eval_gsm_baseline_${curr_timestamp}.out
touch $log_file

for ((i=0; i<${NGC_GPUS_PER_NODE}; i++)); do
    echo "Process: $i run id ${curr_timestamp}"
    CUDA_VISIBLE_DEVICES=${i} nohup ${OI_PY} ${CURR_CODE_PATH}/scripts/custom_eval/gsm/run_eval.py \
    --run_id=${curr_timestamp} \
    --data_dir=data/eval/gsm \
    --save_dir=${results_dir} \
    --model_name_or_path=models/${model_name} \
    --eval_batch_size=20 \
    --n_shot=8 --load_in_8bit \
    --use_wizmath_prompt \
    --world_size=${NGC_GPUS_PER_NODE} --process_idx=${i} 2>&1 >> $log_file &
done

tail -f ${log_file}