#!/bin/bash
set -e
set -x
env_file=$1
source ${env_file}

model_name=$2
task=$3

# LLaMa 2 Paper
# check if task has xxx in 
if [[ "$task" == *"triviaqa"* ]]; then
    num_fewshot=5
elif [[ "$task" == *"gsm8k"* ]]; then
    num_fewshot=8
elif [[ "$task" == *"math_"* ]]; then
    num_fewshot=4
elif [[ "$task" == *"hendrycksTest_"* ]]; then
    num_fewshot=5
elif [[ "$task" == *"bigbench"* ]]; then
    num_fewshot=3
else
    num_fewshot=0
fi

if [[ "$task" == "math_geometry" ]]; then
    batch_size=2
else
    batch_size=4
fi

model_path="${WS_PATH}/exp_artifacts/${model_name}"
if [ ! -d "${model_path}" ]; then
    model_path="${WS_PATH}/models/${model_name}"
    if [ ! -d "${model_path}" ]; then
        echo "Model path ${model_path} does not exist"
        exit 1
    fi
fi

echo "Running LLM Leaderboard for model = $model_name for task = $task"

# Setting the right model args
peft_file="${model_path}/adapter_model.bin"
if [ -f "${peft_file}" ]; then
    echo "Found Peft files"
    if [[ "$model_path" == *"_7b_"* ]]; then
        base_model_path=${WS_PATH}/models/llama2_7b
    elif [[ "$model_path" == *"_13b_"* ]]; then
        base_model_path=${WS_PATH}/models/llama2_13b
    else
        echo "Unknown base model for = $model_path"
        exit 1
    fi
    model_args="pretrained=${base_model_path},peft=${model_path}"
else
    model_args="pretrained=${model_path}"
fi

model_args="${model_args},use_accelerate=True"

results_path="${WS_PATH}/exp_results/${model_name}"
mkdir -p "${results_path}"
results_file="${results_path}/${task}.json"

"${HF_LM_PY}" "${HF_LM_EVAL_PATH}/main.py" \
    --model=hf-causal-experimental \
    --model_args="$model_args" \
    --tasks="${task}" \
    --num_fewshot=${num_fewshot} \
    --batch_size=${batch_size} \
    --output_path=${results_file}\
    --output_base_path=${results_path} \
    --no_cache \
    --write_out
