#!/bin/bash
if [ -z "$1" ]
  then
    echo "No model size supplied"
    exit 1
fi

MODEL_SIZE=$1

EVAL_DATA_PATH=${HOME}/data/eval
DOWLOAD_PATH=${HOME}/data/downloads

MMLU_DATA_PATH=${HOME}/data/eval/mmlu/data
if [ -d "${MMLU_DATA_PATH}" ]; then
    echo "MMLU data already exists"
else
    echo "Downloading MMLU data"
    # MMLU dataset
    wget -O ${HOME}/data/downloads/mmlu_data.tar https://people.eecs.berkeley.edu/~hendrycks/data.tar
    mkdir -p ${HOME}/data/downloads/mmlu_data
    tar -xvf ${HOME}/data/downloads/mmlu_data.tar -C ${HOME}/data/downloads/mmlu_data
    mv ${HOME}/data/downloads/mmlu_data/data ${HOME}/data/eval/mmlu && rm -r ${HOME}/data/downloads/mmlu_data ${HOME}/data/downloads/mmlu_data.tar
fi

# BBH_DATA_PATH=${HOME}/data/eval/bbh/BIG-Bench-Hard-main


# # Big-Bench-Hard dataset
# wget -O ${HOME}/data/downloads/bbh_data.zip https://github.com/suzgunmirac/BIG-Bench-Hard/archive/refs/heads/main.zip
# mkdir -p ${HOME}/data/downloads/bbh
# unzip ${HOME}/data/downloads/bbh_data.zip -d ${HOME}/data/downloads/bbh
# mv ${HOME}/data/downloads/bbh/BIG-Bench-Hard-main/ ${HOME}/data/eval/bbh && rm -r ${HOME}/data/downloads/bbh ${HOME}/data/downloads/bbh_data.zip



# # TyDiQA-GoldP dataset
# mkdir -p ${HOME}/data/eval/tydiqa
# wget -P ${HOME}/data/eval/tydiqa/ https://storage.googleapis.com/tydiqa/v1.1/tydiqa-goldp-v1.1-dev.json
# wget -P ${HOME}/data/eval/tydiqa/ https://storage.googleapis.com/tydiqa/v1.1/tydiqa-goldp-v1.1-train.json


# # GSM dataset
# wget -P ${HOME}/data/eval/gsm/ https://github.com/openai/grade-school-math/raw/master/grade_school_math/data/test.jsonl


# # Codex HumanEval
# wget -P ${HOME}/data/eval/codex_humaneval https://github.com/openai/human-eval/raw/master/data/HumanEval.jsonl.gz

MODEL_NAME=meta_llama_Llama_2_${MODEL_SIZE}b_hf
BASE_MODEL_PATH=${HOME}/models/${MODEL_NAME}

RESULT_PATH=${HOME}/exp_artifacts/$MODEL_NAME/results
mkdir -p ${RESULT_PATH}

echo "Evaluating vanilla model"

export PYTHONPATH=open-instruct

MMLU_RESULT_PATH=${RESULT_PATH}/mmlu
if [ -d "${MMLU_RESULT_PATH}" ]; then
    echo "MMLU results already exist"
else
    echo "Evaluating MMLU"
    mkdir -p ${MMLU_RESULT_PATH}
    python -m eval.mmlu.run_eval \
        --ntrain 0 \
        --data_dir ${MMLU_DATA_PATH} \
        --save_dir ${MMLU_RESULT_PATH} \
        --model_name_or_path ${BASE_MODEL_PATH} \
        --tokenizer_name_or_path ${BASE_MODEL_PATH} \
        --eval_batch_size 4 \
        --load_in_8bit
    echo "Done evaluating MMLU"
fi

echo "Done evaluating"